@@ -1,6 +1,6 @@
 /*
 
-  Copyright 2012-2016 Jochen Topf <jochen@topf.org>.
+  Copyright 2012-2019 Jochen Topf <jochen@topf.org>.
 
   This file is part of OSMCoastline.
 
@@ -15,18 +15,14 @@
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
-  along with OSMCoastline.  If not, see <http://www.gnu.org/licenses/>.
+  along with OSMCoastline.  If not, see <https://www.gnu.org/licenses/>.
 
 */
 
-#include <algorithm>
-#include <cstdlib>
-#include <cstring>
-#include <getopt.h>
-#include <iostream>
-#include <string>
-#include <vector>
+#include "return_codes.hpp"
+#include "version.hpp"
 
+#include <osmium/index/id_set.hpp>
 #include <osmium/io/any_input.hpp>
 #include <osmium/io/error.hpp>
 #include <osmium/io/file.hpp>
@@ -42,10 +38,16 @@
 #include <osmium/util/memory.hpp>
 #include <osmium/util/verbose_output.hpp>
 
-#include "return_codes.hpp"
+#include <algorithm>
+#include <cstdlib>
+#include <cstring>
+#include <getopt.h>
+#include <iostream>
+#include <string>
+#include <vector>
 
 void print_help() {
-    std::cout << "osmcoastline_filter [OPTIONS] OSMFILE\n"
+    std::cout << "Usage: osmcoastline_filter [OPTIONS] OSMFILE\n"
               << "\nOptions:\n"
               << "  -h, --help           - This help message\n"
               << "  -o, --output=OSMFILE - Where to write output (default: none)\n"
@@ -59,17 +61,18 @@ int main(int argc, char* argv[]) {
     bool verbose = false;
 
     static struct option long_options[] = {
-        {"help",         no_argument, 0, 'h'},
-        {"output", required_argument, 0, 'o'},
-        {"verbose",      no_argument, 0, 'v'},
-        {"version",      no_argument, 0, 'V'},
-        {0, 0, 0, 0}
+        {"help",         no_argument, nullptr, 'h'},
+        {"output", required_argument, nullptr, 'o'},
+        {"verbose",      no_argument, nullptr, 'v'},
+        {"version",      no_argument, nullptr, 'V'},
+        {nullptr,                  0, nullptr, 0}
     };
 
-    while (1) {
-        int c = getopt_long(argc, argv, "ho:vV", long_options, 0);
-        if (c == -1)
+    while (true) {
+        const int c = getopt_long(argc, argv, "ho:vV", long_options, nullptr);
+        if (c == -1) {
             break;
+        }
 
         switch (c) {
             case 'h':
@@ -82,9 +85,9 @@ int main(int argc, char* argv[]) {
                 verbose = true;
                 break;
             case 'V':
-                std::cout << "osmcoastline_filter version " OSMCOASTLINE_VERSION "\n"
-                          << "Copyright (C) 2012-2016  Jochen Topf <jochen@topf.org>\n"
-                          << "License: GNU GENERAL PUBLIC LICENSE Version 3 <http://gnu.org/licenses/gpl.html>.\n"
+                std::cout << "osmcoastline_filter " << get_osmcoastline_long_version() << " / " << get_libosmium_version() << '\n'
+                          << "Copyright (C) 2012-2019  Jochen Topf <jochen@topf.org>\n"
+                          << "License: GNU GENERAL PUBLIC LICENSE Version 3 <https://gnu.org/licenses/gpl.html>.\n"
                           << "This is free software: you are free to change and redistribute it.\n"
                           << "There is NO WARRANTY, to the extent permitted by law.\n";
                 std::exit(return_code_ok);
@@ -96,7 +99,7 @@ int main(int argc, char* argv[]) {
     // The vout object is an output stream we can write to instead of
     // std::cerr. Nothing is written if we are not in verbose mode.
     // The running time will be prepended to output lines.
-    osmium::util::VerboseOutput vout(verbose);
+    osmium::util::VerboseOutput vout{verbose};
 
     if (output_filename.empty()) {
         std::cerr << "Missing -o/--output=OSMFILE option\n";
@@ -109,26 +112,28 @@ int main(int argc, char* argv[]) {
     }
 
     osmium::io::Header header;
-    header.set("generator", "osmcoastline_filter");
+    header.set("generator", std::string{"osmcoastline_filter/"} + get_osmcoastline_version());
     header.add_box(osmium::Box{-180.0, -90.0, 180.0, 90.0});
 
     osmium::io::File infile{argv[optind]};
 
+    vout << "Started osmcoastline_filter " << get_osmcoastline_long_version() << " / " << get_libosmium_version() << '\n';
+
     try {
         osmium::io::Writer writer{output_filename, header};
         auto output_it = osmium::io::make_output_iterator(writer);
 
-        std::vector<osmium::object_id_type> ids;
+        osmium::index::IdSetSmall<osmium::object_id_type> ids;
 
         vout << "Reading ways (1st pass through input file)...\n";
         {
             osmium::io::Reader reader{infile, osmium::osm_entity_bits::way};
-            auto ways = osmium::io::make_input_iterator_range<const osmium::Way>(reader);
+            const auto ways = osmium::io::make_input_iterator_range<const osmium::Way>(reader);
             for (const osmium::Way& way : ways) {
                 if (way.tags().has_tag("natural", "coastline")) {
                     *output_it++ = way;
                     for (const auto& nr : way.nodes()) {
-                        ids.push_back(nr.ref());
+                        ids.set(nr.ref());
                     }
                 }
             }
@@ -136,15 +141,15 @@ int main(int argc, char* argv[]) {
         }
 
         vout << "Preparing node ID list...\n";
-        std::sort(ids.begin(), ids.end());
-        auto last = std::unique(ids.begin(), ids.end());
+        ids.sort_unique();
 
         vout << "Reading nodes (2nd pass through input file)...\n";
         {
             osmium::io::Reader reader{infile, osmium::osm_entity_bits::node};
-            auto nodes = osmium::io::make_input_iterator_range<const osmium::Node>(reader);
+            const auto nodes = osmium::io::make_input_iterator_range<const osmium::Node>(reader);
 
-            auto first = ids.begin();
+            auto first = ids.cbegin();
+            const auto last = ids.cend();
             std::copy_if(nodes.cbegin(), nodes.cend(), output_it, [&first, &last](const osmium::Node& node){
                 while (*first < node.id() && first != last) {
                     ++first;
