@@ -1,6 +1,6 @@
 /*
 
-  Copyright 2012-2016 Jochen Topf <jochen@topf.org>.
+  Copyright 2012-2019 Jochen Topf <jochen@topf.org>.
 
   This file is part of OSMCoastline.
 
@@ -15,16 +15,12 @@
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
-  along with OSMCoastline.  If not, see <http://www.gnu.org/licenses/>.
+  along with OSMCoastline.  If not, see <https://www.gnu.org/licenses/>.
 
 */
 
-#include <cstdlib>
-#include <cstring>
-#include <iostream>
-#include <memory>
-#include <string>
-#include <utility>
+#include "return_codes.hpp"
+#include "version.hpp"
 
 #include <osmium/geom/haversine.hpp>
 #include <osmium/geom/ogr.hpp>
@@ -35,23 +31,28 @@
 #include <osmium/io/file.hpp>
 #include <osmium/osm/entity_bits.hpp>
 #include <osmium/osm/location.hpp>
-#include <osmium/osm/way.hpp>
 #include <osmium/osm/types.hpp>
+#include <osmium/osm/way.hpp>
 #include <osmium/visitor.hpp>
 
+#include <gdalcpp.hpp>
+
 #include <ogr_core.h>
 #include <ogr_geometry.h>
 
-#include <gdalcpp.hpp>
-
-#include "return_codes.hpp"
+#include <cstdlib>
+#include <cstring>
+#include <iostream>
+#include <memory>
+#include <string>
+#include <utility>
 
 using index_type = osmium::index::map::SparseMemArray<osmium::unsigned_object_id_type, osmium::Location>;
 using location_handler_type = osmium::handler::NodeLocationsForWays<index_type, index_type>;
 
 class CoastlineWaysHandler : public osmium::handler::Handler {
 
-    double m_length;
+    double m_length = 0.0;
 
     gdalcpp::Dataset m_dataset;
     gdalcpp::Layer m_layer_ways;
@@ -61,7 +62,6 @@ class CoastlineWaysHandler : public osmium::handler::Handler {
 public:
 
     explicit CoastlineWaysHandler(const std::string& db_filename) :
-        m_length(0.0),
         m_dataset("SQLite", db_filename, gdalcpp::SRS{}, {"SPATIALITE=TRUE", "INIT_WITH_EPSG=no" }),
         m_layer_ways(m_dataset, "ways", wkbLineString) {
 
@@ -72,6 +72,12 @@ public:
         m_layer_ways.start_transaction();
     }
 
+    CoastlineWaysHandler(const CoastlineWaysHandler&) = delete;
+    CoastlineWaysHandler operator=(const CoastlineWaysHandler&) = delete;
+
+    CoastlineWaysHandler(CoastlineWaysHandler&&) = delete;
+    CoastlineWaysHandler operator=(CoastlineWaysHandler&&) = delete;
+
     ~CoastlineWaysHandler() {
         m_layer_ways.commit_transaction();
     }
@@ -107,9 +113,9 @@ int main(int argc, char* argv[]) {
         }
 
         if (!std::strcmp(argv[1], "--version") || !std::strcmp(argv[1], "-V")) {
-            std::cout << "osmcoastline_ways version " OSMCOASTLINE_VERSION "\n"
-                      << "Copyright (C) 2012-2016  Jochen Topf <jochen@topf.org>\n"
-                      << "License: GNU GENERAL PUBLIC LICENSE Version 3 <http://gnu.org/licenses/gpl.html>.\n"
+            std::cout << "osmcoastline_ways " << get_osmcoastline_long_version() << " / " << get_libosmium_version() << '\n'
+                      << "Copyright (C) 2012-2019  Jochen Topf <jochen@topf.org>\n"
+                      << "License: GNU GENERAL PUBLIC LICENSE Version 3 <https://gnu.org/licenses/gpl.html>.\n"
                       << "This is free software: you are free to change and redistribute it.\n"
                       << "There is NO WARRANTY, to the extent permitted by law.\n";
             std::exit(return_code_ok);
